@@ -3,6 +3,7 @@ from aiocache.serializers import JsonSerializer
 import pandas as pd
 from .measurementClass import DbMeasurement, FileMeasurement, ComputedMeasurement
 from ..trackhub import TrackHub
+from ..parser import GtfParsedFile, TbxFile
 import ujson
 
 class MeasurementManager(object):
@@ -15,6 +16,7 @@ class MeasurementManager(object):
 
     def __init__(self):
         # self.measurements = pd.DataFrame()
+        self.genomes = {}
         self.measurements = []
 
     def import_dbm(self, dbConn):
@@ -46,7 +48,7 @@ class MeasurementManager(object):
                         )
             self.measurements.append(tempDbM)
 
-    def import_files(self, fileSource, fileHandler=None):
+    def import_files(self, fileSource, fileHandler=None, genome=None):
         """Import measurements from a file. 
 
 
@@ -63,8 +65,15 @@ class MeasurementManager(object):
             if "annotation" in rec["datatype"]:
                 isGene = True
 
+            if rec.get("genome") is None and genome is None: 
+                raise Exception("all files must be annotated with its genome build")
+
+            tgenome = rec.get("genome")
+            if tgenome is None:
+                tgenome = genome
+
             tempFileM = FileMeasurement(rec.get("file_type"), rec.get("id"), rec.get("name"), 
-                            rec.get("url"), annotation=rec.get("annotation"),
+                            rec.get("url"), genome=tgenome, annotation=rec.get("annotation"),
                             metadata=rec.get("metadata"), minValue=0, maxValue=5,
                             isGenes=isGene, fileHandler=fileHandler
                         )
@@ -89,7 +98,7 @@ class MeasurementManager(object):
                 measurements.append(tempFile)
         return measurements
 
-    def add_computed_measurement(self, mtype, mid, name, measurements, computeFunc, annotation=None, metadata=None, computeAxis=1):
+    def add_computed_measurement(self, mtype, mid, name, measurements, computeFunc, genome=None, annotation=None, metadata=None, computeAxis=1):
         """Add a Computed Measurement
 
         Args: 
@@ -103,28 +112,45 @@ class MeasurementManager(object):
             a `ComputedMeasurement` object
         """
         
-        tempComputeM = ComputedMeasurement(mtype, mid, name, measurements=measurements, computeFunc=computeFunc, annotation=annotation, metadata=metadata, computeAxis=computeAxis)
+        tempComputeM = ComputedMeasurement(mtype, mid, name, measurements=measurements, computeFunc=computeFunc, genome=genome, annotation=annotation, metadata=metadata, computeAxis=computeAxis)
         self.measurements.append(tempComputeM)
         return tempComputeM
 
-
-    def add_genome(self, genome, fileHandler=None, url="http://obj.umiacs.umd.edu/genomes/"):
+    def add_genome(self, genome, type, url="http://obj.umiacs.umd.edu/genomes/", fileHandler=None):
         """Add a genome to the list of measurements. The genome has to be tabix indexed for the file server
            to make remote queries. Our tabix indexed files are available at https://obj.umiacs.umd.edu/genomes/index.html
 
         Args: 
-            genome: for example : hg19
+            genome: for example : hg19 if type = "tabix" or full location of gtf file if type = "gtf"
+            genome_id: required if type = "gtf"
             url: url to the genome file
         """
         isGene = True
+        tempGenomeM = None
 
-        gurl = url + genome + "/" + genome + ".txt.gz"
-        tempGenomeM = FileMeasurement("tabix", genome, genome, 
-                        gurl, annotation={"group": "genome"},
-                        metadata=["geneid", "exons_start", "exons_end", "gene"], minValue=0, maxValue=5,
-                        isGenes=isGene, fileHandler=fileHandler, columns=["chr", "start", "end", "width", "strand", "geneid", "exon_starts", "exon_ends", "gene"]
-                    )
-        self.measurements.append(tempGenomeM)
+        if type is "tabix":
+            gurl = url + genome + "/" + genome + ".txt.gz"
+            tempGenomeM = FileMeasurement("tabix", genome, genome, 
+                            gurl, genome, annotation={"group": "genome"},
+                            metadata=["geneid", "exons_start", "exons_end", "gene"], minValue=0, maxValue=5,
+                            isGenes=isGene, fileHandler=fileHandler, columns=["chr", "start", "end", "width", "strand", "geneid", "exon_starts", "exon_ends", "gene"]
+                        )
+            # self.genomes.append(tempGenomeM)
+            # gtf_file = TbxFile(gurl)
+            # self.genomes[genome] = gtf_file
+            self.measurements.append(tempGenomeM)
+        elif type is "gtf":
+            gurl = url
+            tempGenomeM = FileMeasurement("gtfparsed", genome, genome, 
+                            gurl, genome=genome, annotation={"group": "genome"},
+                            metadata=["geneid", "exons_start", "exons_end", "gene"], minValue=0, maxValue=5,
+                            isGenes=isGene, fileHandler=fileHandler, columns=["chr", "start", "end", "width", "strand", "geneid", "exon_starts", "exon_ends", "gene"]
+                        )
+
+            gtf_file = GtfParsedFile(gurl)
+            self.genomes[genome] = gtf_file
+            self.measurements.append(tempGenomeM)
+            
         return(tempGenomeM)
 
     def get_measurements(self):
@@ -132,6 +158,11 @@ class MeasurementManager(object):
         """
         return self.measurements
 
+    def get_genomes(self):
+        """Get all available genomes
+        """
+        return self.genomes
+
     def import_trackhub(self, hub, handler=None):
         """Import measurements from annotationHub objects. 
 
