@@ -154,30 +154,33 @@ class BigWig(BaseFile):
 
         if bins is None:
             bins = 2000
-        
-        bins = (end - start) if bins > (end - start) else bins
-        # basesPerBin = (end - start)*1.0/bins if zoomlvl is -1 else 0
-        if zoomlvl != -2:
-            zoomlvl, zoomOffset = self.getZoom(zoomlvl, (end - start) / bins)
-        else:
-            self.zooms = {}
-            zoomOffset = self.header.get("fullIndexOffset")
 
-        # if not self.tree.get(str(zoomlvl)):
-        #     self.sync = True
-        #     self.tree[str(zoomlvl)] = self.getTree(zoomlvl)
+        try:
+            bins = (end - start) if bins > (end - start) else bins
+            # basesPerBin = (end - start)*1.0/bins if zoomlvl is -1 else 0
+            if zoomlvl != -2:
+                zoomlvl, zoomOffset = self.getZoom(zoomlvl, (end - start) / bins)
+            else:
+                self.zooms = {}
+                zoomOffset = self.header.get("fullIndexOffset")
 
-        values = self.getValues(chr, start, end, zoomlvl)
+            # if not self.tree.get(str(zoomlvl)):
+            #     self.sync = True
+            #     self.tree[str(zoomlvl)] = self.getTree(zoomlvl)
 
-        # result = values
-        if respType is "DataFrame":
-            result = toDataFrame(values, self.columns)
-            # replace chrmId with chr
-            result["chr"] = chr
+            values = self.getValues(chr, start, end, zoomlvl)
 
-        if self.sync:
-            return result, {"zooms": self.zooms, "chrmIds": self.chrmIds, "tree": self.tree, "cacheData": self.cacheData}
-        return result, None
+            # result = values
+            if respType is "DataFrame":
+                result = toDataFrame(values, self.columns)
+                # replace chrmId with chr
+                result["chr"] = chr
+
+            if self.sync:
+                return result, {"zooms": self.zooms, "chrmIds": self.chrmIds, "tree": self.tree, "cacheData": self.cacheData}
+            return result, None
+        except Exception as e:
+            return pd.DataFrame(columns = self.columns), str(e)
 
     # a note on zoom levels: 0 to totalLevels are the regular zoom level index
     # -2 for using fullDataOffset
@@ -409,14 +412,16 @@ class BigWig(BaseFile):
         """
         offset = node.get("rOffset")
 
-        if self.cacheData.get(str(offset)):
-            tree = self.cacheData.get(str(offset))
+        if self.cacheData.get(str(zoomlvl) + "-" + str(offset)):
+            tree = self.cacheData.get(str(zoomlvl) + "-" + str(offset))
         else:
             if node.get("rIsLeaf"):
+                # print("leaf")
                 tree = self.getTreeBytes(zoomlvl, offset, node.get("rCount") * 32)
             else:
+                # print("not leaf")
                 tree = self.getTreeBytes(zoomlvl, offset, node.get("rCount") * 24)
-            self.cacheData[str(offset)] = tree
+            self.cacheData[str(zoomlvl) + "-" + str(offset)] = tree
 
         if node.get("rIsLeaf"):
             for i in range(0, node.get("rCount")):
@@ -635,7 +640,7 @@ class BigWig(BaseFile):
         for i in range(0, itemCount):
             if zoomlvl is not -2:
                 (chromId, startv, endv, validCount, minVal, maxVal, sumData, sumSquares) = struct.unpack("4I4f", decom[i*32 : (i+1)*32])
-                valuev = sumData / validCount
+                valuev = (sumData/validCount) if validCount > 0 else sumData
             elif iType == 1:
                 (startv, endv, valuev) = struct.unpack(self.endian + "IIf", decom[24 + 12*i : 24 + 12*(i+1)])
             elif iType == 2:
@@ -649,4 +654,4 @@ class BigWig(BaseFile):
             if endv >= start and startv <= end and chromId == chrmId:
                 result.append((chromId, startv, endv, valuev))
 
-        return result
\ No newline at end of file
+        return result
